package organization;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Stream;

import util.Gender;

/**
 * This class represents an employee in a supervisory role. This means that this
 * employee supervises at least one other employee
 */
public class Supervisor extends GenericEmployee {

  private List<Employee> supervisee;

  public Supervisor(String name, double pay, Gender gender) {
    super(name, pay, gender);
    supervisee = new LinkedList<Employee>();
  }

  public List<Employee> getSupervisees() {
    return Collections.unmodifiableList(supervisee);
  }


  @Override
  public Employee addSupervisee(String supervisorName, Employee supervisee) {
    if (this.name.equals(supervisorName)) {
      this.supervisee.add(supervisee);
      return this;
    }
    for (int i = 0; i < this.supervisee.size(); i++) {
      this.supervisee.set(
              i,
              this.supervisee.get(i).addSupervisee(supervisorName,
                      supervisee));
    }
    return this;
  }

  @Override
  public <R> R accept(EmployeeVisitor<R> visitor) {
    return visitor.visitSupervisor(this);
  }
}
