package organization;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * This visitor converts the organization hierarchy to a list of employees
 */
public class ToListVisitor implements EmployeeVisitor<List<Employee>> {
  @Override
  public List<Employee> visitContractEmployee(ContractEmployee e) {
    return visitSingle(e);
  }

  @Override
  public List<Employee> visitInternalEmployee(InternalEmployee e) {
    return visitSingle(e);
  }

  @Override
  public List<Employee> visitNonManagerEmployee(NonManagerEmployee e) {
    return visitSingle(e);
  }

  @Override
  public List<Employee> visitSupervisor(Supervisor e) {
    return visitComposite(e);
  }

  private List<Employee> visitSingle(NonManagerEmployee e) {
    return Arrays.asList(e);
  }

  private List<Employee> visitComposite(Supervisor e) {
    List<Employee> list = new ArrayList<Employee>();
    list.add(e);
    for (Employee emp:e.getSupervisees()) {
      list.addAll(emp.accept(this));
    }
    return list;
  }
}
